"""This is a blind proxy that we use to get around browser
restrictions that prevent the Javascript from loading pages not on the
same server as the Javascript.  This has several problems: it's less
efficient, it might break some sites, and it's a security risk because
people can use this proxy to browse the web and possibly do bad stuff
with it.  It only loads pages via http and https, but it can load any
content type. It supports GET and POST requests."""

import urllib.request, urllib.error, urllib.parse
import sys, os

# Designed to prevent Open Proxy type stuff.

def proxy_method(request):

    allowedHosts = ['www.openlayers.org', 'openlayers.org',
                    'localhost','localhost:8080',
                    'ows.terrestris.de', 'ows-tile.terrestris.de']

    method = os.environ.get("REQUEST_METHOD", "")

    status = None
    response = None
    contentType = None

    if method == "POST":
        qs = os.environ.get("QUERY_STRING", "")
        d = urllib.parse.parse_qs(qs)
        if "url" in d:
            url = d["url"][0]
        else:
            url = "http://www.openlayers.org"
    else:
        url = request.GET.get("url", "http://www.openlayers.org")

    try:
        url = urllib.parse.unquote(url)
        host = url.split("/")[2]
        if allowedHosts and not host in allowedHosts:
            status = 502
            contentType = 'Content-Type: text/plain'
            response = 'This proxy does not allow you to access that location (%s).' % (host)

        elif url.startswith("http://") or url.startswith("https://"):

            if method == "POST":
                length = int(os.environ.get("CONTENT_LENGTH", ""))
                headers = {"Content-Type": os.environ.get("CONTENT_TYPE", "")}
                body = sys.stdin.read(length)
                r = urllib.request.Request(url, body, headers)
                y = urllib.request.urlopen(r)
            else:
                y = urllib.request.urlopen(url)

            # print content type header
            i = y.info()
            if "Content-Type" in i:
                contentType = "Content-Type: %s" % (i["Content-Type"])
            else:
                contentType = "Content-Type: text/plain"

            response = y.read()

            y.close()
        else:
            contentType = "Content-Type: text/plain"
            response = "Illegal request."

    except Exception as E:
        status = 500
        contentType = "Content-Type: text/plain"
        response = "Some unexpected error occurred. Error text was:", E

    return (status, contentType, response)