/*global Cesium, Ext, ExtACe*/
/*jslint plusplus: true, sloppy: true, white: true*/

/**


 Published under the BSD license.

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 Neither the name of the terrestris GmbH & Co. KG nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

 @author terrestris GmbH & Co. KG
 */

/**
 * @class
 */
Ext.define('ExtACe.widgets.LoginWindow', {
    extend: 'Ext.window.Window',
    xtype: 'w_login_window',
    requires:[
        'ExtACe.parser.Config'
    ],

    title: null,

    name: null,

    width: null,

    layout: 'fit',

    autoScroll: true,

    collapsible: false,

    resizable: false,

    constrainHeader: true,

    modal: true,

    /**
     *
     */
    viewer: null,

    /**
     *
     */
    initComponent : function() {
        var me = this;

        // create the window items
        me.createLoginWindowItems();

        // register custom keyboard navigation
        me.on('afterrender', me.createKeyNav);

        // call super
        me.callParent();

    },

    /**
     *
     */
    createKeyNav: function() {
        var me = this;

        Ext.create('Ext.util.KeyNav', me.getEl(), {
            scope: me,
            enter: me.login
        });

    },

    /**
     * Create the items for the window
     */
    createLoginWindowItems: function() {
        var me = this;

        me.items = [{
            xtype: 'form',
            border: true,
            bodyPadding: 10,
            width: 300,
            defaultType: 'textfield',
            items: [{
                name: 'loginname',
                fieldLabel: 'Name',
                allowBlank: false
            }, {
                inputType: 'password',
                name: 'loginpass',
                fieldLabel: 'Passwort',
                allowBlank: false
            }],
            bbar: [{
                xtype: 'tbfill'
            }, {
                xtype: 'button',
                text: 'Anmelden',
                handler: function() {
                    var formPanel = this.up('form');
                    me.login(formPanel);
                }
            }]
        }];

    },

    /**
     *
     */
    login: function() {
        var me = this,
            formPanel = me.down('form'),
            values = formPanel.getValues();

        if (formPanel.isValid()) {
            Ext.Ajax.request({
                url: '/django/auth/login/',
                params: {
                    user: values.loginname,
                    pass: values.loginpass
                },
                success: function(result, request){
                    var jsonData = Ext.JSON.decode(result.responseText),
                        resultMessage = jsonData.msg,
                        resultParam = jsonData.para,
                        welcomeName;

                    //Set ExtACe.user
                    ExtACe.user = jsonData.user;

                    if (resultMessage === 'Login OK') {
                        formPanel.getForm().reset();
                        me.close();

                        welcomeName = jsonData.user.firstname || jsonData.user.username

                        Ext.Msg.alert('Login erfolgreich',
                                      'Hallo ' + welcomeName  + '. <br> Dein GPExt wird jetzt gestartet.',
                                      function(){
                                          location.reload();
                        });
                    }
                    if (resultMessage === 'Fail') {
                        Ext.Msg.alert('Login fehlgeschlagen', 'Bitte wiederholen Sie den Login!');
                    }
                    if (resultMessage === 'FailInactive') {
                        Ext.Msg.alert('Login fehlgeschlagen', 'Ihr Account ist nicht aktiviert!');
                    }

                },
                failure: function(){
                    Ext.Msg.alert('Login fehlgeschlagen', 'Bitte wiederholen Sie den Login!');
                }
            });
        }
    },

    /**
     *
     */
    reloadGrid: function(){
        var gridPanel = Ext.ComponentQuery.query('w_track_grid')[0],
            gridStore = gridPanel.getStore();

        gridStore.reload();
    }
});
