/*global Cesium, Ext, ExtACe*/
/*jslint plusplus: true, sloppy: true, white: true*/

/**


 Published under the BSD license.

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 Neither the name of the terrestris GmbH & Co. KG nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

 @author terrestris GmbH & Co. KG
 */

/**
 * @class
 */
Ext.define('ExtACe.widgets.NominatimCombo', {
    extend : 'Ext.form.ComboBox',
    xtype : 'w_nominatim_combo',
    requires : [
        'Ext.XTemplate',
        'Ext.data.reader.Json'
    ],

    /**
     * api: config[map] ``OpenLayers.Map or Object`` A configured map or a
     * configuration object for the map constructor, required only if
     * :attr:`zoom` is set to value greater than or equal to 0.
     */
    /**
     * private: property[map] ``OpenLayers.Map`` The map object.
     */
    viewer: null,

    /**
     * api: config[width] See
     * http://www.dev.sencha.com/deploy/dev/docs/source/BoxComponent.html#cfg-Ext.BoxComponent-width,
     * default value is 350.
     */
    width: 350,

    /**
     * api: config[listWidth] See
     * http://www.dev.sencha.com/deploy/dev/docs/source/Combo.html#cfg-Ext.form.ComboBox-listWidth,
     * default value is 350.
     */
    listWidth: 350,

    /**
     * api: config[loadingText] See
     * http://www.dev.sencha.com/deploy/dev/docs/source/Combo.html#cfg-Ext.form.ComboBox-loadingText,
     * default value is "Search in Geonames...".
     */
    loadingText: 'Bitte warten...',

    /**
     * api: config[emptyText] See
     * http://www.dev.sencha.com/deploy/dev/docs/source/TextField.html#cfg-Ext.form.TextField-emptyText,
     * default value is "Search location in Geonames".
     */
    emptyText: 'Ortssuche...',

    /**
     * api: config[minChars] ``Number`` Minimum number of characters to be typed
     * before search occurs, defaults to 1.
     */
    minChars: 1,

    /**
     * api: config[queryDelay] ``Number`` Delay before the search occurs,
     * defaults to 500 ms.
     */
    queryDelay: 500,

    /**
     * api: config[maxRows] `String` The maximum number of rows in the
     * responses, defaults to 20, maximum allowed value is 1000. See:
     * http://www.geonames.org/export/geonames-search.html
     */
    /**
     * private: property[maxRows] ``String``
     */
    maxRows: '20',

    /**
     * api: config[tpl] ``Ext.XTemplate or String`` Template for presenting the
     * result in the list (see
     * http://www.dev.sencha.com/deploy/dev/docs/output/Ext.XTemplate.html), if
     * not set a default value is provided.
     */
    //tpl: '<tpl for="."><div class="x-combo-list-item"><h1>{name}</div></tpl>',

    /*******************************************************************************
     * STANDARD GEOCODING
     ******************************************************************************/

    /**
     * api: config[format] Output format format=[html|xml|json]
     */
    /**
     * private: property[format] ``String``
     */
    format: 'json',

    /**
     * api: config[accept-language] Preferred language order for showing search
     * results, overrides the browser value. Either uses standard rfc2616
     * accept-language string or a simple comma separated list of language
     * codes.
     */
    /**
     * private: property[accept-language] ``String``
     */
    // COMMENT JW: Leads to Syntax error
    // accept-language: '',

    /**
     * api: config[viewbox] viewbox=<left>,<top>,<right>,<bottom> or
     * viewboxlbrt=<left>,<bottom>,<right>,<top> The preferred area to find
     * search results
     */
    /**
     * private: property[viewbox] ``String``
     */
     viewbox: '-180,90,180,-90',

    /**
     * api: config[bounded] bounded=[0|1] Restrict the results to only items
     * contained with the bounding box. Restricting the results to the
     * bounding box also enables searching by amenity only, for example a
     * search query of just "[pub]" would normally be rejected but with
     * bounded=1 will result in a list of items matching within the bounding
     * box.
     */
    /**
     * private: property[bounded] ``Number``
     */
     bounded: 0,

    /**
     * api: config[polygon] polygon=[0|1] Output polygon outlines for items
     * found
     */
    /**
     * private: property[polygon] ``Number``
     */
     polygon: 0,

    /**
     * api: config[addressdetails] addressdetails=[0|1] Include a breakdown
     * of the address into elements
     */
    /**
     * private: property[addressdetails] ``Number``
     */
    addressdetails: 1,

    /**
     * api: config[exclude_place_ids] exclude_place_ids=<place_id,[place_id],[place_id]>
     * If you do not want certain openstreetmap objects to appear in the search
     * result, give a comma separated list of the place_id's you want to skip.
     */
    /**
     * private: property[exclude_place_ids] ``Number``
     */
    exclude_place_ids: '',

   /**
     * api: config[limit] limit=<integer> Limit the number of returned results.
     */
    /**
     * private: property[limit] ``Number``
     */
    limit: 999,

    /**
     * api: config[countrycodes] countrycodes=<ISO 3166-1 alpha2 code> Limit
     * search results to a specific country (or a list of countries).
     * <countrycode> should be the ISO 3166-1 alpha2 code, e.g. gb for the uk,
     * de for germany, etc. http://en.wikipedia.org/wiki/ISO_3166-1
     */
    /**
     * private: property[countrycodes] ``String``
     */
    countrycodes: '',

    /**
     * city=<city>
     */
    city: '',

    /*******************************************************************************
     * END OF STANDARD GEOCODING
     ******************************************************************************/




    /*******************************************************************************
     * REVERSE GEOCODING
     ******************************************************************************/

    /**
     * api: config[accept-language] Preferred language order for showing search
     * results, overrides the browser value. Either uses standard rfc2616
     * accept-language string or a simple comma separated list of language
     * codes.
     */
    /**
     * private: property[accept-language] ``String``
     */
    // COMMENT JW: Leads to Syntax error
    // accept-language: '',

    /**
     * api: config[osm_type] osm_type=[N|W|R] A specific osm node / way /
     * relation to return an address for Please use this in preference to
     * lat/lon where possible
     */
    /**
     * private: property[osm_type] ``String``
     */
    osm_type: '',

    /**
     * api: config[osm_id] osm_id=<value> A specific osm id to return an
     * address for Please use this in preference to lat/lon where possible
     */
    /**
     * private: property[osm_id] ``String``
     */
    osm_id: '',

    /**
     * api: config[zoom] Level of detail required where 0 is country and 18 is
     * house/building
     */
    /**
     * private: property[zoom] ``Number``
     */
    zoom: 0,

    /**
     * api: config[lat] Latitude of Position for reverse geocoding
     */
    /**
     * private: property[lat] ``Number``
     */
    lat: '',

    /**
     * api: config[lon] Longitude of Position for reverse geocoding
     */
    /**
     * private: property[lon] ``Number``
     */
    lon: '',

    /**
     * api: config[country_code] ``String`` Countrycode (ISO2) in which to make
     * a search, default is all countries.
     */
    /**
     * private: property[country_code] ``String``
     */
    country_code: '',

    /**
     * api: config[country] ``String`` Country in which to make a search,
     * default is all countries.
     */
    /**
     * private: property[country] ``String``
     */
    country: '',

    /*******************************************************************************
     * END OF REVERSE GEOCODING
     ******************************************************************************/

    /**
     * private: property[hideTrigger] Hide trigger of the combo.
     */
    hideTrigger: false,

    /**
     * private: property[displayField] Display field name
     */
    displayField: 'name',

    /**
     * private: property[forceSelection] Force selection.
     */
    forceSelection: true,

    /**
     * private: property[queryParam] Query parameter.
     */
    queryParam: 'q',

    /**
     * private: property[url] Url of the GeoNames service:
     * http://www.GeoNames.org/export/GeoNames-search.html
     */
    url: 'http://nominatim.openstreetmap.org/search?',

    /**
     *
     */
    initComponent : function() {
        var me = this;

        if (me.viewer === null) {
            me.viewer = ExtACe.widgets.ViewerBox.getViewer();
        }

        me.callParent(arguments);

        Ext.define('NominatimResponse', {
            extend: 'Ext.data.Model',
            fields: [{
                name: 'name',
                mapping: 'display_name'
            }, {
                name: 'lon',
                mapping: 'lon'
            }, {
                name: 'lat',
                mapping: 'lat'
            }, {
                name: 'bbox',
                mapping: 'boundingbox'
            }, {
                name: 'osm_id',
                mapping: 'osm_id'
            }, {
                name: 'station',
                mapping: 'address.station'
            }, {
                name: 'city',
                mapping: 'address.city'
            }, {
                name: 'road',
                mapping: 'address.road'
            }, {
                name: 'neighbourhood',
                mapping: 'address.neighbourhood'
            }, {
                name: 'county',
                mapping: 'address.county'
            }, {
                name: 'state',
                mapping: 'address.state'
            }, {
                name: 'postcode',
                mapping: 'address.postcode'
            }]
        });

        me.store = Ext.create('Ext.data.Store', {
            model: 'NominatimResponse',
            proxy: Ext.create('Ext.data.ScriptTagProxy', {
                url: me.url,
                method: 'GET',
                callbackKey: 'json_callback',
                extraParams: {
                    format: me.format,
                    viewbox: me.viewbox,
                    bounded: me.bounded,
                    polygon: me.polygon,
                    addressdetails: me.addressdetails,
                    exclude_place_ids: me.exclude_place_ids,
                    limit: me.limit,
                    countrycodes: me.countrycodes,
                    osm_type: me.osm_type,
                    osm_id: me.osm_id,
                    lat: me.lat,
                    lon: me.lon,
                    zoom: me.zoom,
                    city: me.city
                },
                reader: {
                    type: 'json'
                }
            })
        });

        me.on("select", function(combo, records, index) {
            var destination,
                flight;

            destination = Cesium.Cartographic.fromDegrees(records[0].data.lon, records[0].data.lat, 15000);

            flight = Cesium.CameraFlightPath.createAnimationCartographic(me.viewer.scene, {
                destination : destination
            });

            me.viewer.scene.getAnimations().add(flight);
        }, me);

    }
});
