/*global Cesium, Ext, ExtACe*/
/*jslint plusplus: true, sloppy: true, white: true*/

/**


 Published under the BSD license.

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 Neither the name of the terrestris GmbH & Co. KG nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

 @author terrestris GmbH & Co. KG
 */

/**
 * @class
 */
Ext.define('ExtACe.widgets.RegistrationWindow', {
    extend: 'Ext.window.Window',
    xtype: 'w_registration_window',

    title: null,

    name: null,

    width: null,

    layout: 'fit',

    autoScroll: true,

    collapsible: false,

    resizable: false,

    constrainHeader: true,

    /**
     *
     */
    viewer: null,

    /**
     *
     */
    initComponent : function() {
        var me = this;

        // create the window items
        me.createRegistrationWindowItems();

        // call super
        me.callParent();

    },

    /**
     * Create the items for the window.
     */
    createRegistrationWindowItems: function() {
        var me = this;

        me.items = [{
            xtype: 'form',
            border: true,
            bodyPadding: 10,
            defaultType: 'textfield',
            defaults: {
                labelWidth: 150
            },
            items: [{
                fieldLabel: 'Vorname',
                name: 'regvorname',
                allowBlank: false
            }, {
                fieldLabel: 'Nachname',
                name: 'regnachname',
                allowBlank: false
            }, {
                fieldLabel: 'Benutzername',
                name: 'regbenutzername',
                allowBlank: false
            }, {
                fieldLabel: 'Passwort',
                inputType: 'password',
                name: 'regpasswort',
                itemId: 'pass',
                allowBlank: false,
                listeners: {
                    validitychange: function(field){
                        field.next().validate();
                    },
                    blur: function(field){
                        field.next().validate();
                    }
                }
            }, {
                fieldLabel: 'Passwort bestätigen',
                inputType: 'password',
                name: 'regpassconfirm',
                vtype: 'password',
                initialPassField: 'pass' // id of the initial password field
            }, {
                fieldLabel: 'E-Mailadresse',
                name: 'regemail',
                vtype: 'email',
                allowBlank: false
            }],
            bbar: [{
                xtype: 'tbfill'
            }, {
                text: 'Anmelden',
                handler: function() {
                    var formPanel = this.up('form');
                    me.register(formPanel);
                }
            },{
                text: 'Abbrechen',
                handler: function(){
                    me.close();
                }
            }]
        }];

        // Validation of the passwordfields
        Ext.apply(Ext.form.field.VTypes, {
            password: function(val, field) {
                if (field.initialPassField) {
                    var pwd = field.up('form').down('#' + field.initialPassField);
                    return (val == pwd.getValue());
                }
                return true;
            },
            passwordText: 'Passwörter stimmen nicht überein!'
        });

    },

    /**
     * Sends the registration form to the backend.
     * @param {Ext.form.Panel} formPanel
     */
    register: function(formPanel) {
        var me = this,
            values = formPanel.getValues();

        if (formPanel.isValid()) {
            Ext.Ajax.request({
                url: 'django/auth/registration/',
                params: {
                    vorname: values.regvorname,
                    nachname: values.regnachname,
                    benutzername: values.regbenutzername,
                    email: values.regemail,
                    passwort: values.regpasswort
                },
                success: function(result, request){

                    var jsonData = Ext.JSON.decode(result.responseText);
                    var resultMessage = jsonData.msg;
                    var resultParam = jsonData.para;
                    if (resultMessage === 'OK') {
                        Ext.Msg.alert('Anmeldung erfolgreich', 'Sie können sich nun einloggen.');
                        formPanel.getForm().reset();
                        me.close();
                    }
                    if (resultMessage === 'Fail') {
                        Ext.Msg.alert('Anmeldung fehlgeschlagen', 'Anmeldung fehlgeschlagen. Bitte wiederholen Sie die Anmeldung!');
                    }
                    if (resultMessage === 'Faildoubleuser') {
                        Ext.Msg.alert('Anmeldung fehlgeschlagen', 'Anmeldung fehlgeschlagen. Ein Benutzer mit dieser Emailadresse/ diesem Namen exisitert bereits!');
                    }
                },
                failure: function(){
                    Ext.Msg.alert('Anmeldung fehlgeschlagen', 'Anmeldung fehlgeschlagen. Bitte wiederholen Sie die Anmeldung!');
                }
            });
        }
    }
});