/*global Cesium, Ext, ExtACe*/
/*jslint plusplus: true, sloppy: true, white: true*/

/**


 Published under the BSD license.

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 Neither the name of the terrestris GmbH & Co. KG nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

 @author terrestris GmbH & Co. KG
 */

/**
 * @class
 */
Ext.define('ExtACe.widgets.TrackGridPanel', {
    extend : 'Ext.grid.Panel',
    xtype: 'w_track_grid',
    requires : [
    ],

    layout: 'fit',

    /**
     *
     */
    getAllTracksUrl: '/django/getTracks',

    /**
     *
     */
    getTrackDetailsUrl: '/django/getTrackDetails',

    /**
     *
     */
    deleteTrackUrl: '/django/deleteTrack',

    /**
     *
     */
    createCzmlUrl: '/django/createCzml',

    /**
     *
     */
    panelsRendered: false,

    store: null,

    columns: null,

    viewer: null,

    autoScroll: true,

    /**
     *
     */
    initComponent : function() {
        var me = this;

        if (me.viewer === null) {
            me.viewer = ExtACe.widgets.ViewerBox.getViewer();
        }

        me.columns = [
            {text: 'Name', dataIndex: 'name', flex: 1},
            {text: 'Aktivität',  dataIndex: 'type', flex: 1}
        ];

        me.store = Ext.create('Ext.data.Store', {
            fields: [
                {name: 'pk', type: 'int'},
                {name: 'name', type: 'string', mapping: 'fields.name'},
                {name: 'desc', type: 'string', mapping: 'fields.desc'},
                {name: 'type',  type: 'string', mapping: 'fields.type'}
            ],
            proxy: {
                type: 'ajax',
                url: me.getAllTracksUrl,
                reader: {
                    root: 'data',
                    type: 'json'
                }
            },
            autoLoad: true
        });

        me.on("itemdblclick", function(grid, record) {
            me.createTrack(record.get('pk'));
        }, me);

        me.on("itemclick", function(grid, record) {
            me.getTrackDetails(record.get('pk'));
        }, me);

        me.on("beforeitemcontextmenu", function(view, record, item, index, e){
              e.stopEvent();
              var menu = me.createGridMenu();
                  menu.clickedRecord = record;
                  menu.showAt(e.getXY());
        })

        me.callParent(arguments);
    },

    /**
     *
     */
    createGridMenu: function(){
        var me = this,
            grid = Ext.create('Ext.menu.Menu', {
                items: [{
                    text: 'Anzeigen',
                    name: 'm_show'
                },{
                    text: 'Löschen',
                    name: 'm_delete'
                },{
                    text: 'Einstellungen',
                    name: 'm_settings'
                }],
                listeners:{
                    click: function(menu, item, e, eOpts){
                        var record = this.clickedRecord,
                            trackId = record.data.pk;

                        switch(item.name){
                        case 'm_show':
                            me.createTrack(trackId);
                            break;
                        case 'm_delete':
                            Ext.Msg.confirm('Track löschen',
                                    'Wollen sie den Track wirklich löschen?',
                                    function(button){
                                        if(button==="yes"){
                                            me.deleteTrack(trackId)
                                        }
                                    }
                            );
                            break;
                        case 'm_settings':
                            me.getTrackDetails(trackId);
                            break;
                        }
                    }
                }
            });

        return grid
    },

    /**
     *
     */
    deleteTrack: function(trackId) {
        var me = this;

           Ext.Ajax.request({
               url: me.deleteTrackUrl,
               method: 'POST',
               params: {
                   trkid: trackId
               },
               success: function(response){
                   var responseText = response.responseText,
                       responseJson = Ext.JSON.decode(responseText),
                       czmlData = responseJson.data

                       Ext.Msg.alert('Track gelöscht!',responseJson.msg)
                       me.getStore().reload();

                       console.log('TODO: Delete CZML from globe!')
               }
           });
   },

    /**
     *
     */
    getTrackDetails: function(trackId) {
       var me = this;

       Ext.Ajax.request({
           url: me.getTrackDetailsUrl,
           method: 'POST',
           params: {
               trkid: trackId
           },
           success: function(response) {
               var responseText = response.responseText;

               try {
                   var responseJson = Ext.JSON.decode(responseText),
                       trackData = responseJson.data[0];

                   me.updateTrackDetailPanels(trackData);
               }
               catch(e) {
                   Ext.create('ExtACe.utils.MsgWindow', {
                       msg: 'Fehler beim Laden der Track-Details.',
                       errorText: e
                   });
               }

           },
           failure: function(response) {
               Ext.create('ExtACe.utils.MsgWindow', {
                   msg: 'Fehler beim Laden der Track-Details.',
                   errorText: response.responseText
               });
           }
       });
    },

    /**
     *
     */
    updateTrackDetailPanels: function(trackData) {
        var me = this,
            fields = trackData.fields,
            southPanels = Ext.ComponentQuery.query('panel[region=south]'),
            profileChart = Ext.ComponentQuery.query('chart')[0],
            profileChartStore = profileChart.getStore(),
            propGrid = Ext.ComponentQuery.query('propertygrid')[0];

        // show south panels (hidden by default)
        Ext.each(southPanels, function(panel) {
            if (panel.isHidden()) {
                panel.show()
            }
        });

        // load profile data
        profileChartStore.load({
            params: {
                trkid: trackData.pk
            }
        });

        // set the profile max value
        profileChart.axes.getAt(1).maximum = fields.length

        // load propertygrid data
        propGrid.setSource({
            trackId: trackData.pk,
            name: fields.name,
            description: fields.desc,
            activity: fields.type,
            length: Math.round(fields.length) + " m"
        }, {
            trackId: {
                displayName: "Track ID",
                editor: new Ext.form.DisplayField()
            },
            name: {
                displayName: "Name"
            },
            description: {
                displayName: "Beschreibung"
            },
            activity:{
                displayName: "Aktivität",
                editor: new Ext.form.field.ComboBox({
                    store: ['bike', 'car', 'hike', 'paddeling',
                            'running', 'skiing'],
                    forceSelection: true
                })
            },
            length: {
                displayName: "Länge",
                editor: new Ext.form.DisplayField()
            }
        });

    },

    /**
     *
     */
    createTrack: function(trackId) {
        var me = this;

        Ext.Ajax.request({
            url: me.createCzmlUrl,
            method: 'POST',
            params: {
                trkid: trackId
            },
            success: function(response){
                var responseText = response.responseText,
                    responseJson =Ext.JSON.decode(responseText),
                    czmlData = responseJson.data

                me.addCzmlData(czmlData);
                var viewer = ExtACe.widgets.ViewerBox.getViewer()
            }
        });
    },

    /**
     *
     */
    addCzmlData: function(data) {
        var me = this,
            scene = me.viewer.scene,
            czmlDataSource = new Cesium.CzmlDataSource(),
            destination = Cesium.Cartographic.fromDegrees(7.138854, 50.742019, 15000.0),
            position = {};

            position.lon = data[0].position.cartographicDegrees[1]; //stupid czml-syntax
            position.lat = data[0].position.cartographicDegrees[2]; //stupid czml-syntax

            if(position.lon && position.lat){
                destination = Cesium.Cartographic.fromDegrees(position.lon, position.lat, 15000.0);
            }

            var flight = Cesium.CameraFlightPath.createAnimationCartographic(scene, {
            destination : destination
        });

        scene.getAnimations().add(flight);

        me.viewer.dataSources.removeAll();

        czmlDataSource.load(data, 'GPX-Track');
        me.viewer.dataSources.add(czmlDataSource);
    }

});