/*global Cesium, Ext, ExtACe*/
/*jslint plusplus: true, sloppy: true, white: true*/

/**


 Published under the BSD license.

 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 Neither the name of the terrestris GmbH & Co. KG nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

 @author terrestris GmbH & Co. KG
 */

/**
 * @class
 */
Ext.define('ExtACe.widgets.TrackProfilePanel', {
    extend : 'Ext.panel.Panel',
    xtype : 'w_track_profile',
    requires : [
        'Ext.fx.target.Sprite'
    ],

    layout: 'fit',

    border: false,

    viewer: null,

    autoScroll: false,

    /**
     *
     */
    getTrackProfileUrl: '/django/getTrackProfile',

    /**
     *
     */
    initComponent : function() {
        var me = this;

        if (me.viewer === null) {
            me.viewer = ExtACe.widgets.ViewerBox.getViewer();
        }
        me.items = me.createProfileChart();

        me.callParent(arguments);
    },

    /**
     *
     */
    createProfileChart: function() {
        var me = this,
            chart;

        chart = Ext.create('Ext.chart.Chart', {
            height: 150,
            animate: false,
            name: 'profile-name',
            shadow: false,
            store: me.createProfileChartStore(),
            axes: me.createChartAxis(),
            series: me.createProfileChartSeries()
        });

        me.viewer.startTime = new Cesium.JulianDate.fromIso8601("2010-01-01T00:00:00Z")

        me.viewer.clock.onTick.addEventListener(
                function(){
                    if(chart.series.get(1).items){
                        // Remove addHours when timeparsing in gpxpy is fixed
                        var currentTime = Ext.Date.format(Ext.Date.parse(me.viewer.clock.currentTime.addHours(1).toIso8601(), 'Y-m-dTH:i:s.u\\Z'), 'Y-m-d\\TH:i'),
                            matchingItem = chart.getStore().find('timeStr',currentTime),
                            series = chart.series.get(1);

                        if(matchingItem >= 0){
                            series.unHighlightItem();
                            series.highlightItem(series.items[matchingItem]);
                        }
                    }
                }
        )

        return chart;
    },

    /**
     *
     */
    createChartAxis: function() {
        var me = this,
            leftAxis, bottomAxis,
            axis = [];

        leftAxis = {
            title: 'Höhe über NN',
            type: 'Numeric',
            position: 'left',
            fields: ['elv'],
            grid: true,
            label: {
                renderer: Ext.util.Format.numberRenderer('0,0')
            }
        };

        bottomAxis = {
            type: 'Numeric',
            position: 'bottom',
            fields: ['dist']
        };

        axis.push(leftAxis, bottomAxis);

        return axis;
    },

    /**
     *
     */
    createProfileChartSeries: function() {
        var me = this,
            series;

        series = [{
            type: 'area',
            axis: 'left',
            xField: 'dist',
            yField: 'elv',
            highlight: true,
            renderer: function(sprite, storeItem, barAttr, i, store) {
                barAttr.fill = 'orange';
                barAttr.opacity = 0.7;
                return barAttr;
            },
            tips: {
                trackMouse: true,
                renderer: function(storeItem, item) {
                    this.update(
                            'Höhe: ' + Math.round(storeItem.data.elv) + ' m <br>' +
                            'Zeitpunkt: ' + storeItem.data.time.toUTCString()
                    );
                }
            },
            listeners: {
                itemclick: function (param) {
                    var clickedTime = param.storeItem.data.timeStr;
                    me.viewer.clock.currentTime = Cesium.JulianDate.fromIso8601(clickedTime);
                }
            }
        },{
            type: 'scatter',
            axis: 'left',
            xField: 'dist',
            yField: 'elv',
            selectionTolerance: 0,
            highlight: {
                radius: 5,
                fill: '#3892D3',
                'stroke-width': 0
            },
            markerConfig: {
                size: 0,
                radius: 0
            }
        }];

        return series;
    },

    /**
     *
     */
    createProfileChartStore: function() {
        var me = this,
            store;

        store = Ext.create('Ext.data.Store', {
            autoLoad: false,
            fields: [
                {name: 'dist', type: 'float', mapping: 'fields.dist'},
                {name: 'elv',  type: 'float',  mapping: 'fields.elv'},
                {name: 'time', type: 'date', dateFormat: 'Y-m-dTH:i:s', mapping: 'fields.time'},
                {name: 'timeStr', type: 'string', mapping: 'fields.time'}
            ],
            proxy: {
                type: 'ajax',
                url: me.getTrackProfileUrl,
                actionMethods: {
                    create: 'POST',
                    read: 'POST',
                    update: 'POST',
                    destroy: 'POST'
                },
                reader: {
                    type: 'json',
                    root: 'data'
                },
                listeners: {
                    exception: function (proxy, response, operation) {
                        Ext.Msg.show({
                            title: response.statusText,
                            msg: 'Fehler beim Laden der Profildaten: <br/>' + response.responseText,
                            maxHeight: 300
                        });
                    }
                }
            }
        });

        return store;
    }

});